/* This file was generated by syncqt. */
#ifndef QT_QTPRINTSUPPORT_VERSION_H
#define QT_QTPRINTSUPPORT_VERSION_H

#define QTPRINTSUPPORT_VERSION_STR "6.11.2"

#define QTPRINTSUPPORT_VERSION 0x0601102

#endif // QT_QTPRINTSUPPORT_VERSION_H
